/*
 *  Authors: Rodney Dawes <dobey.pwns@gmail.com>
 *
 *  Copyright 2008 Rodney Dawes
 *
 */

#ifdef _HAVE_CONFIG_H_
#include "config.h"
#endif

#include "calview-month.h"

#include <glib/gi18n.h>
#include <time.h>

struct _CalViewMonthPrivate {
};

/* GObject methods */
static void cal_view_month_finalize (GObject * object);

/* CalViewModel methods */
static void cal_view_month_expose (CalViewModel * model,
				   GtkWidget * widget,
				   GdkEventExpose * event);

G_DEFINE_TYPE (CalViewMonth, cal_view_month, CAL_TYPE_VIEW_MODEL)

static void cal_view_month_class_init (CalViewMonthClass * klass) {
  GObjectClass * o_class;
  CalViewModelClass * parent_class;

  o_class = (GObjectClass *) klass;
  parent_class = (CalViewModelClass *) klass;

  /* GObject Methods */
  o_class->finalize = cal_view_month_finalize;

  /* CalViewModel methods */
  parent_class->expose_event = cal_view_month_expose;
}

static void cal_view_month_init (CalViewMonth * model) {
  CalViewMonthPrivate * priv;

  priv = g_new0 (CalViewMonthPrivate, 1);
  model->priv = priv;
}

static void cal_view_month_finalize (GObject * object) {
  CalViewMonth * model = CAL_VIEW_MONTH (object);

  if (model->priv)
    g_free (model->priv);
}

static void cal_view_month_expose (CalViewModel * model,
				   GtkWidget * widget,
				   GdkEventExpose * event) {
  cairo_t * context;
  cairo_text_extents_t text;
  int bw, bh;
  int i;
  time_t tnow;
  struct tm * tmnow;
  int daysinmonth[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  int days;
  int week, tweek;
  int wday;
  int th;
  int tdw;
  gboolean use_abbr = FALSE;

  int wx, wy, ww, wh;

  tnow = time (NULL);
  tmnow = localtime (&tnow);

  wx = 0 + 1; //widget->allocation.x;
  wy = widget->allocation.y + 1;
  ww = widget->allocation.width - 2;
  wh = widget->allocation.height - 2;

  days = daysinmonth[tmnow->tm_mon];
  /* If February in leap year, add a day */
  if (tmnow->tm_mon == 1 && ((tmnow->tm_year + 1900) % 4) == 0)
    days = 29;

  context = gdk_cairo_create (widget->window);

  cairo_set_tolerance (context, 0.0f);
  cairo_set_operator (context, CAIRO_OPERATOR_OVER);
  cairo_set_line_width (context, 1.0f);

  cairo_rectangle (context, event->area.x, event->area.y,
		   event->area.width, event->area.height);
  cairo_clip (context);

  cairo_set_source_rgba (context, 1.0f, 1.0f, 1.0f, 1.0f);
  cairo_rectangle (context, wx, wy, ww, wh);
  cairo_fill (context);

  cairo_text_extents (context, "Wednesday", &text);
  th = text.height;

  cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.05f);
  cairo_rectangle (context, wx, wy, ww, th + 6.5);
  cairo_fill (context);

  cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.2f);
  cairo_move_to (context, wx, th + 6.5);
  cairo_line_to (context, wx + ww, th + 6.5);
  cairo_stroke (context);

  bw = widget->allocation.width / 7;
  bh = (widget->allocation.height - 6 - th) / 6;

  if (bw < text.width + 12)
    use_abbr = TRUE;

  for (i = 1; i < 8; i++) {
    if (i < 7) {
      cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.2f);
      cairo_move_to (context, (bw * i) + 0.5 + wx, th + 6);
      cairo_line_to (context, (bw * i) + 0.5 + wx, wh);
      cairo_stroke (context);
    }

    cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 1.0f);
    switch (i) {
    case 1:
      cairo_text_extents (context, use_abbr ? _("Sun") : _("Sunday"), &text);
      cairo_move_to (context, (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Sun") : _("Sunday"));
      break;
    case 2:
      cairo_text_extents (context, use_abbr ? _("Mon") : _("Monday"), &text);
      cairo_move_to (context, bw * (i - 1) + (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Mon") : _("Monday"));
      break;
    case 3:
      cairo_text_extents (context, use_abbr ? _("Tue") : _("Tuesday"), &text);
      cairo_move_to (context, bw * (i - 1) + (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Tue") : _("Tuesday"));
      break;
    case 4:
      cairo_text_extents (context, use_abbr ? _("Wed") : _("Wednesday"), &text);
      cairo_move_to (context, bw * (i - 1) + (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Wed") : _("Wednesday"));
      break;
    case 5:
      cairo_text_extents (context, use_abbr ? _("Thu") : _("Thursday"), &text);
      cairo_move_to (context, bw * (i - 1) + (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Thu") : _("Thursday"));
      break;
    case 6:
      cairo_text_extents (context, use_abbr ? _("Fri") : _("Friday"), &text);
      cairo_move_to (context, bw * (i - 1) + (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Fri") : _("Friday"));
      break;
    case 7:
      cairo_text_extents (context, use_abbr ? _("Sat") : _("Saturday"), &text);
      cairo_move_to (context, bw * (i - 1) + (bw / 2) - (text.width / 2) + 0.5 + wx,
		     (th + 6) / 2 + 2.5);
      cairo_show_text (context, use_abbr ? _("Sat") : _("Saturday"));
      break;
    }
  }

  for (i = 1; i < 7; i++) {
    if (i < 6) {
      cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.2f);
      cairo_move_to (context, wx, (bh * i) + (th + 6) + 0.5);
      cairo_line_to (context, wx + ww, (bh * i) + (th + 6) + 0.5);
      cairo_stroke (context);
    }
    cairo_set_source_rgba (context, 0.0f, 0.0f, 1.0f, 0.1f);
    cairo_move_to (context, wx, (bh * (i - 1)) + (th + 6) + 0.5);
    cairo_rectangle (context, wx, (bh * (i - 1)) + (th + 6) + 0.5,
		     ww, th + 6 + 6.5);
    cairo_fill (context);
  }

  tweek = 0;
  wday = tmnow->tm_wday;

  for (i = tmnow->tm_mday; i > 0; i--) {
    wday--;
    if (wday < 0) {
      wday = 6;
      tweek++;
    }
  }

  if (tmnow->tm_wday == 6)
    tdw = ww - (6 * bw);
  else
    tdw = bw;

  cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.215f);
  cairo_rectangle (context,
		   tmnow->tm_wday * bw + wx,
		   (th + 6) + (bh * tweek),
		   tdw, bh);
  cairo_fill (context);

  week = tweek;
  wday = tmnow->tm_wday;

  cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 1.0f);
  for (i = tmnow->tm_mday; i > 0; i--) {
    char * label;

    label = g_strdup_printf ("%d", i);
    cairo_text_extents (context, label, &text);
    cairo_move_to (context, bw * wday + (bw - text.width - 6) + wx,
		   (th + 16) + (bh * week) + (th  + 6) / 2 + 0.5);
    cairo_show_text (context, label);
    g_free (label);

    wday--;
    if (wday < 0) {
      wday = 6;
      week--;
    }
    if (week < 0)
      break;
  }
  if (wday != 0 || week == 0) {
    cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.08f);
    cairo_rectangle (context, wx, (th + 6.5),
		     bw * (wday + 1), bh);
    cairo_fill (context);
  }

  week = tweek;
  wday = tmnow->tm_wday + 1;

  if (wday > 6) {
    wday = 0;
    week++;
  }

  cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 1.0f);
  for (i = tmnow->tm_mday + 1; i <= days; i++) {
    char * label;

    label = g_strdup_printf ("%d", i);
    cairo_text_extents (context, label, &text);
    cairo_move_to (context, bw * wday + (bw - text.width - 6) + wx,
		   (th + 16) + (bh * week) + (th  + 6) / 2 + 0.5);
    cairo_show_text (context, label);
    g_free (label);

    wday++;
    if (wday > 6) {
      wday = 0;
      week++;
    }
    if (week > 6)
      break;
  }

  cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.08f);
  if (wday <= 6) {
    cairo_set_source_rgba (context, 0.0f, 0.0f, 0.0f, 0.08f);
    cairo_rectangle (context, bw * wday + wx,
		     (th + 6) + bh * week,
		     ww, bh);
    cairo_fill (context);
  }
  if (week < 5) {
    cairo_rectangle (context, wx, (th + 6) + bh * 5,
		     ww, bh);
    cairo_fill (context);
  }

  cairo_destroy (context);

  /* Just Temporary - Will get a shadow from the ScrolledWindow instead */
  gtk_paint_shadow (widget->style, widget->window, GTK_STATE_NORMAL,
		    GTK_SHADOW_IN, &event->area, widget, "calview",
		    0, 0, widget->allocation.width, widget->allocation.height);
}

/* Public methods */
CalViewModel * cal_view_month_new (void) {
  return CAL_VIEW_MODEL (g_object_new (CAL_TYPE_VIEW_MONTH, NULL));
}
