/*
 *  Authors: Rodney Dawes <dobey.pwns@gmail.com>
 *
 *  Copyright 2008 Rodney Dawes
 *
 */

#ifdef _HAVE_CONFIG_H_
#include "config.h"
#endif

#include "calview.h"

#include <gtk/gtkbuildable.h>

struct _CalViewPrivate {
  CalViewModel * model;

  GtkWidget * area;
};

/* GObject Methods */
static void cal_view_finalize (GObject * object);

/* GtkObject Methods */
static void cal_view_destroy (GtkObject * object);

/* GtkWidget Methods */
static gboolean cal_view_expose (GtkWidget * widget,
				 GdkEventExpose * event);
static void cal_view_style_set (GtkWidget * widget,
				GtkStyle * old_style);

/* GtkBuildable Methods */
static void cal_view_buildable_init (GtkBuildableIface * iface);
static void cal_view_buildable_add_child (GtkBuildable * cal_view,
					  GtkBuilder * builder,
					  GObject * child,
					  const gchar * type);

G_DEFINE_TYPE_WITH_CODE (CalView, cal_view, GTK_TYPE_DRAWING_AREA,
			 G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
						cal_view_buildable_init))

static void cal_view_class_init (CalViewClass * klass) {
  GObjectClass * o_class;
  GtkObjectClass * obj_class;
  GtkWidgetClass * wid_class;
  GtkDrawingAreaClass * parent_class;

  o_class = (GObjectClass *) klass;
  obj_class = (GtkObjectClass *) klass;
  wid_class = (GtkWidgetClass *) klass;
  parent_class = (GtkDrawingAreaClass *) klass;

  /* GObject Methods */
  o_class->finalize = cal_view_finalize;

  /* GtkObject Methods */
  obj_class->destroy = cal_view_destroy;

  /* GtkWidget Methods */
  wid_class->expose_event = cal_view_expose;
  wid_class->style_set = cal_view_style_set;
}

static void cal_view_buildable_init (GtkBuildableIface * iface) {
  iface->add_child = cal_view_buildable_add_child;
}

static void cal_view_init (CalView * cal_view) {
  CalViewPrivate * priv;

  priv = g_new0 (CalViewPrivate, 1);
  cal_view->priv = priv;

  GTK_WIDGET_SET_FLAGS (cal_view, GTK_CAN_FOCUS);
}

static void cal_view_finalize (GObject * object) {
  CalView * view;
  CalViewPrivate * priv;

  view = CAL_VIEW (object);
  priv = view->priv;

  if (priv->model)
    g_object_unref (priv->model);

  g_free (priv);

  if (G_OBJECT_CLASS (cal_view_parent_class)->finalize)
    (* G_OBJECT_CLASS (cal_view_parent_class)->finalize) (object);
}

static void cal_view_buildable_add_child (GtkBuildable * cal_view,
					  GtkBuilder * builder,
					  GObject * child,
					  const gchar * type) {
}

/* GtkObject Methods */
static void cal_view_destroy (GtkObject * object) {
  if (GTK_OBJECT_CLASS (cal_view_parent_class)->destroy)
    (* GTK_OBJECT_CLASS (cal_view_parent_class)->destroy) (object);
}

/* GtkWidget Methods */
static gboolean cal_view_expose (GtkWidget * widget,
				 GdkEventExpose * event) {
  CalView * view;
  CalViewPrivate * priv;

  view = CAL_VIEW (widget);
  priv = view->priv;

  if (priv->model && CAL_IS_VIEW_MODEL (priv->model)) {
    CalViewModelClass * mclass = CAL_VIEW_MODEL_GET_CLASS (priv->model);

    if (mclass->expose_event)
      mclass->expose_event (priv->model, widget, event);

    return FALSE;
  }

  return TRUE;
}

static void cal_view_style_set (GtkWidget * widget,
				GtkStyle * old_style) {
}

/* Creators */
GtkWidget * cal_view_new (void) {
  return GTK_WIDGET (g_object_new (CAL_TYPE_VIEW, NULL));
}

GtkWidget * cal_view_new_with_model (CalViewModel * model) {
  CalViewPrivate * priv;
  GtkWidget * calview;

  g_return_val_if_fail (model != NULL, NULL);

  calview = cal_view_new ();
  priv = CAL_VIEW (calview)->priv;

  priv->model = model;

  return calview;
}
