/*
 *  Authors: Rodney Dawes <dobey.pwns@gmail.com>
 *
 *  Copyright 2008 Rodney Dawes
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include "calview.h"
#include "calview-month.h"

static void rhetorical_quit (GtkWidget * widget, gpointer data) {
  gtk_main_quit ();
}

static gboolean rhetorical_delete_event (GtkWidget * widget,
					 GdkEventAny * event,
					 gpointer data) {
  rhetorical_quit (widget, data);

  return TRUE;
}

static gboolean rhetorical_create (void * data) {
  GtkWidget * window;
  GtkWidget * calview;
  CalViewModel * model;
  GtkWidget * pane;
  GtkWidget * scroll;

  gtk_window_set_default_icon_name ("rhetorical");

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Calendar");
  gtk_window_set_default_size (GTK_WINDOW (window), 640, 480);
  gtk_widget_realize (window);

  pane = gtk_hpaned_new ();
  gtk_paned_set_position (GTK_PANED (pane), 128);
  gtk_container_add (GTK_CONTAINER (window), pane);
  gtk_widget_show (pane);

  scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_paned_add1 (GTK_PANED (pane), scroll);
  gtk_widget_show (scroll);

  model = cal_view_month_new ();

  calview = cal_view_new_with_model (model);
  gtk_paned_add2 (GTK_PANED (pane), calview);
  gtk_widget_show (calview);

  g_signal_connect (window, "delete-event",
		    G_CALLBACK (rhetorical_delete_event), NULL);

  gtk_widget_show (window);

  return FALSE;
}

gint main (gint argc, gchar ** argv) {
  gtk_init (&argc, &argv);

  g_idle_add (rhetorical_create, NULL);

  gtk_main ();

  return 0;
}
